### Listing 2.1

```scala
import play.api.mvc._

class Application extends Controller {
  def tweets = Action {
    Ok
  }
}
```

### Listing 2.2

```
# Routes
# This file defines all application routes
# (Higher priority routes first)
# ~~~~

# Home page
GET     /                controllers.Application.index
GET     /tweets          controllers.Application.tweets

# Map static resources from the /public folder to the /assets URL path
GET     /assets/*file    controllers.Assets.at(path="/public", file)
```

### Listing 2.3

```scala
import play.api.libs.oauth.{ConsumerKey, RequestToken}
import play.api.Play.current
import scala.concurrent.Future
import play.api.libs.concurrent.Execution.Implicits._

def tweets = Action.async {                                   
  val credentials: Option[(ConsumerKey, RequestToken)] = for {
    apiKey <- Play.configuration.getString("twitter.apiKey")
    apiSecret <- Play.configuration.getString("twitter.apiSecret")
    token <- Play.configuration.getString("twitter.token")
    tokenSecret <- Play.configuration.getString("twitter.tokenSecret")
  } yield (
      ConsumerKey(apiKey, apiSecret),
      RequestToken(token, tokenSecret)
    )

  credentials.map { case (consumerKey, requestToken) =>
    Future.successful {                                                 
      Ok
    }
  } getOrElse {
    Future.successful {                                                 
      InternalServerError("Twitter credentials missing")      
    }
  }
}
```

### Listing 2.4

```scala
import play.api.libs.oauth.{ConsumerKey, RequestToken}
import play.api.Play.current
import scala.concurrent.Future
import play.api.libs.concurrent.Execution.Implicits._
import play.api.libs.ws._

def tweets = Action.async {
  credentials.map { case (consumerKey, requestToken) =>
    WS
      .url("https://stream.twitter.com/1.1/statuses/filter.json") 
      .sign(OAuthCalculator(consumerKey, requestToken))             
      .withQueryString("track" -> "reactive")                         
      .get()                                                        
      .map { response =>
        Ok(response.body)
      }
    } getOrElse {
      Future.successful {
        InternalServerError("Twitter credentials missing")
      }
    }
}

def credentials: Option[(ConsumerKey, RequestToken)] = for {
  apiKey <- Play.configuration.getString("twitter.apiKey")
  apiSecret <- Play.configuration.getString("twitter.apiSecret")
  token <- Play.configuration.getString("twitter.token")
  tokenSecret <- Play.configuration.getString("twitter.tokenSecret")
} yield (
  ConsumerKey(apiKey, apiSecret),
  RequestToken(token, tokenSecret)
)
```

### Listing 2.5

```scala
import play.api.libs.oauth.{ConsumerKey, RequestToken}
import play.api.Play.current
import scala.concurrent.Future
import play.api.libs.concurrent.Execution.Implicits._
import play.api.libs.ws._
import play.api.libs.iteratee._
import play.api.Logger

def tweets = Action.async {

  val loggingIteratee = Iteratee.foreach[Array[Byte]] { array => 
    Logger.info(array.map(_.toChar).mkString)
  }

  credentials.map { case (consumerKey, requestToken) =>
    WS
      .url("https://stream.twitter.com/1.1/statuses/filter.json")
      .sign(OAuthCalculator(consumerKey, requestToken))
      .withQueryString("track" -> "reactive")
      .get { response =>                                   
        Logger.info("Status: " + response.status)
        loggingIteratee                                     
      }.map { _ =>
        Ok("Stream closed")                                 
      }
}

def credentials = ...
```

### Listing 2.6

```
resolvers += "Typesafe private" at
  "https://private-repo.typesafe.com/typesafe/maven-releases"

libraryDependencies +=
  "com.typesafe.play.extras" %% "iteratees-extras" % "1.5.0"
```

### Listing 2.7

```scala
import play.api.libs.oauth.{ConsumerKey, RequestToken}
import play.api.Play.current
import scala.concurrent.Future
import play.api.libs.concurrent.Execution.Implicits._
import play.api.libs.ws._
import play.api.libs.iteratee._
import play.api.Logger
import play.api.libs.json._
import play.extras.iteratees._

def tweets = Action.async {
    credentials.map { case (consumerKey, requestToken) =>
      val (iteratee, enumerator) = Concurrent.joined[Array[Byte]] 

      val jsonStream: Enumerator[JsObject] =                       
        enumerator &>
        Encoding.decode() &>
        Enumeratee.grouped(JsonIteratees.jsSimpleObject)

      val loggingIteratee = Iteratee.foreach[JsObject] { value =>
        Logger.info(value.toString)
      }

      jsonStream run loggingIteratee                             

      WS
        .url("https://stream.twitter.com/1.1/statuses/filter.json")
        .sign(OAuthCalculator(consumerKey, requestToken))
        .withQueryString("track" -> "reactive")
        .get { response =>
          Logger.info("Status: " + response.status)
          iteratee                                             
        }.map { _ =>
          Ok("Stream closed")
        }
    }
}

def credentials = ...
```

### Listing 2.8

```scala
package actors

import akka.actor.{Actor, ActorRef, Props}
import play.api.Logger
import play.api.libs.json.Json

class TwitterStreamer(out: ActorRef) extends Actor {
  def receive = {                                         
    case "subscribe" =>                                  
      Logger.info("Received subscription from a client")
      out ! Json.obj("text" -> "Hello, world!")            
  }
}

object TwitterStreamer {
  def props(out: ActorRef) = Props(new TwitterStreamer(out)) 
}
```

### Listing 2.9

```
// ...
import actors.TwitterStreamer

// ...
def tweets = WebSocket.acceptWithActor[String, JsValue] {
  request => out => TwitterStreamer.props(out)
}

```
### Listing 2.10

```
@(message: String)(implicit request: RequestHeader)

@main(message) {
    <div id="tweets"></div>                             
    <script type="text/javascript">
        var url = "@routes.Application.tweets().webSocketURL()";
        var tweetSocket = new WebSocket(url);               

        tweetSocket.onmessage = function (event) {              
            console.log(event);
            var data = JSON.parse(event.data);
            var tweet = document.createElement("p");
            var text = document.createTextNode(data.text);
            tweet.appendChild(text);
            document.getElementById("tweets" ).appendChild(tweet);
        };

        tweetSocket.onopen = function() {                     
            tweetSocket.send("subscribe");                     
        };
    </script>
}
```

### Listing 2.11

```scala
def index = Action { implicit request =>
  Ok(views.html.index("Tweets"))
}
```

### Listing 2.12

```scala
object TwitterStreamer {
  def props(out: ActorRef) = Props(new TwitterStreamer(out))

  private var broadcastEnumerator: Option[Enumerator[JsObject]] = None 

  def connect(): Unit = {
    credentials.map { case (consumerKey, requestToken) =>
      val (iteratee, enumerator) = Concurrent.joined[Array[Byte]]      

      val jsonStream: Enumerator[JsObject] = enumerator &>            
        Encoding.decode() &>
        Enumeratee.grouped(JsonIteratees.jsSimpleObject)

      val (be, _) = Concurrent.broadcast(jsonStream)                
      broadcastEnumerator = Some(be)

      val url = "https://stream.twitter.com/1.1/statuses/filter.json"
      WS
        .url(url)
        .sign(OAuthCalculator(consumerKey, requestToken))
        .withQueryString("track" -> "reactive")
        .get { response =>
          Logger.info("Status: " + response.status)
          iteratee                                                  
        }.map { _ =>
          Logger.info("Twitter stream closed")
        }

    } getOrElse {
      Logger.error("Twitter credentials missing")
    }
  }
}
```

### Listing 2.13

```scala
object TwitterStreamer {

  // ...

  def subscribe(out: ActorRef): Unit = {
    if (broadcastEnumerator.isEmpty) {
      connect()
    }
    val twitterClient = Iteratee.foreach[JsObject] { t => out ! t }
    broadcastEnumerator.foreach { enumerator =>
      enumerator run twitterClient
    }
}
```
### Listing 2.14

```scala
class TwitterStreamer(out: ActorRef) extends Actor {
  def receive = {
    case "subscribe" =>
      Logger.info("Received subscription from a client")
      TwitterStreamer.subscribe(out)
  }
}
```

### Listing 2.15

```
function appendTweet(text) {
    var tweet = document.createElement("p");
    var message = document.createTextNode(text);
    tweet.appendChild(message);
    document.getElementById("tweets").appendChild(tweet);
}

function connect(attempt) {                                
    var connectionAttempt = attempt;
    var url = "@routes.Application.tweets().webSocketURL()";
    var tweetSocket = new WebSocket(url);
    tweetSocket.onmessage = function (event) {
        console.log(event);
        var data = JSON.parse(event.data);
        appendTweet(data.text);
    };
    tweetSocket.onopen = function() {
        connectionAttempt = 1;
        tweetSocket.send("subscribe");
    };
    tweetSocket.onclose = function() {                      
        if (connectionAttempt <= 3) {                         
            appendTweet("WARNING: Lost server connection,
	attempting to reconnect. Attempt number " + connectionAttempt);
            setTimeout(function() {                            
                connect(connectionAttempt + 1);                    
            }, 5000);
        } else {
            alert("The connection with the server was lost.");       
        }
    };
}

connect(1);                                                     
```

### Listing 2.16

```
def subscribeNode: Enumerator[JsObject] = {
  if (broadcastEnumerator.isEmpty) {
    connect()
  }
  broadcastEnumerator.getOrElse {
    Enumerator.empty[JsObject]
  }
}
```

### Listing 2.17

```scala
class Application extends Controller {

  // ...

  def replicateFeed = Action { implicit request =>
    Ok.feed(TwitterStreamer.subscribeNode)
  }
}
```
