## Listing 5.1

```
val response: Future[WSResponse] =
  WS.url("http://www.playframework.com").get() 

val siteOnline: Future[Boolean] = response.map { r => 
  r.status == 200
}

siteOnline.foreach { isOnline => 
  if(isOnline) {
    println("The Play site is up")
  } else {
    println("The Play site is down")
  }
}
```

### Listing 5.2

```
val response: Future[WSResponse] =
  WS.url("http://www.playframework.com").get()

val siteAvailable: Future[Option[Boolean]] = response.map { r => 
  Some(r.status == 200)
  } recover { 
    case ce: java.net.ConnectException => None 
  }
```

### Listing 5.3

```
def siteAvailable(url: String): Future[Boolean] = 
  WS.url(url).get().map { r =>
    r.status == 200
}

val playSiteAvailable = 
  siteAvailable("http://www.playframework.com")
val playGithubAvailable = 
  siteAvailable("https://github.com/playframework")

val allSitesAvailable: Future[Boolean] = for { 
  siteAvailable <- playSiteAvailable
  githubAvailable <- playGithubAvailable
} yield (siteAvailable && githubAvailable) 
```

### Listing 5.4

```
import scala.concurrent._
import java.util.concurrent.Executors

implicit val ec = ExecutionContext.fromExecutor( 
  Executors.newFixedThreadPool(2) 
)

val sum: Future[Int] = Future { 1 + 1 } 
sum.foreach { s => println(s) }
```

### Listing 5.5

```
import scala.concurrent._
import scala.concurrent.ExecutionContext.Implicits.global
import java.io.File

def fileExists(path: String): Future[Boolean] = Future {
  new java.io.File(path).exists  
} 
```

### Listing 5.6

```
import play.api.libs.ws._
import scala.concurrent._
import play.api.libs.concurrent.Execution.Implicits._
import play.api.Play.current

def availability = Action.async { 
  val response: Future[WSResponse] =
    WS.url("http://www.playframework.com").get()
  val siteAvailable: Future[Boolean] = response.map { r =>
    r.status == 200
  }
  siteAvailable.map { isAvailable => 
    if(isAvailable) {
      Ok("The Play site is up.")
    } else {
      Ok("The Play site is down!")  
    }
  }
}
```

### Listing 5.7

```
def authenticationErrorHandler: PartialFunction[Throwable, Result] = { 
  case UserNotFoundException(userId) => 
    NotFound(
      Json.obj("error" -> s"User with ID $userId was not found")
    )
  case UserDisabledException(userId) => 
    Unauthorized(
      Json.obj("error" -> s"User with ID $userId is disabled")
    )
  case ce: ConnectionException => 
    ServiceUnavailable(
      Json.obj("error" -> "Authentication backend broken")
    )
}

val authentication: Future[Result] = ???  

val recoveredAuthentication: Future[Result] =
  authentication.recover(authenticationErrorHandler) 
```

### Listing 5.8

```
import play.api.libs.concurrent.Promise
import scala.concurrent.duration._

case class AuthenticationResult(success: Boolean, error: String)

def authenticate(username: String, password: String) = Action.async {
  implicit request =>
    val authentication: Future[AuthenticationResult] =
      authenticationService.authenticate(username, password)
    val timeoutFuture = Promise.timeout(
      "Authentication service unresponsive", 2.seconds
    ) 
    Future.firstCompletedOf(
      Seq(authentication, timeoutFuture)
    ).map { 
      case AuthenticationResult(success, _) if success =>
        Ok("You can pass")
      case AuthenticationResult(success, error) if !success =>
        Unauthorized(s"You shall not pass: $error")
      case timeoutReason: String =>
        ServiceUnavailable(timeoutReason)
    }
}
```

### Listing 5.9

```
contexts {
    graph-db {
        fork-join-executor { 
          parallelism-max = 2 
        }
    }
    image-resizer {
        fork-join-executor {
            parallelism-factor = 10.0 
            parallelism-max = 50
        }
    }
}
```

### Listing 5.10

```
object Contexts {
  val graphDb: ExecutionContext =
    Akka.system.dispatchers.lookup("contexts.graph-db")
  val imageResizer: ExecutionContext =
    Akka.system.dispatchers.lookup("contexts.image-resizer")
}
```

### Listing 5.11

```
def complexReport: Future[Report] = Future { 
  val reportData = queryGraphDb() 
  makeReport(reportData)
}(Contexts.graphDb) 
```

### Listing 5.12

```
import scala.concurrent.duration._

class AuthenticationServiceSpec extends Specification {

  "The AuthenticationService" should {
    val service = new DefaultAuthenticationService

    "correctly authenticate Bob Marley" in {
      implicit ee: ExecutionEnv => 
        service.authenticateUser("bob@marley.org", "secret")
        must beEqualTo (AuthenticationSuccessful).await(1, 200.millis) 
    }

    "not authenticate Ziggy Marley" in { implicit ee: ExecutionEnv =>
      service.authenticateUser("ziggy@marley.org", "secret")
      must beEqualTo (AuthenticationUnsuccessful).await(1, 200.millis)
    }

    "fail if it takes too long" in { implicit ee: ExecutionEnv =>
      service.authenticateUser("jimmy@hendrix.com", "secret")
      must throwA[RuntimeException].await(1, 600.millis) 
    }

  }
}
```

### Listing 5.13

```
trait StatisticsRepository {

  def storeCounts(counts: StoredCounts)
    (implicit ec: ExecutionContext): Future[Unit] 

  def retrieveLatestCounts(userName: String)
    (implicit ec: ExecutionContext): Future[StoredCounts] 

}

case class StoredCounts( 
  when: DateTime,
  userName: String,
  followersCount: Long,
  friendsCount: Long
)
```

### Listing 5.14

```
trait TwitterService {
  
  def fetchRelationshipCounts(userName: String)
    (implicit ec: ExecutionContext): Future[TwitterCounts] 

  def postTweet(message: String)
    (implicit ec: ExecutionContext): Future[Unit] 

}

case class TwitterCounts(followersCount: Long, friendsCount: Long) 
```

### Listing 5.15

```
trait StatisticsService {
  def createUserStatistics(userName: String)
    (implicit ec: ExecutionContext): Future[Unit]
}
```

### Listing 5.16

```
class DefaultStatisticsService(
  statisticsRepository: StatisticsRepository,
  twitterService: TwitterService) extends StatisticsService {

  override def createUserStatistics(userName: String)
    (implicit ec: ExecutionContext): Future[Unit] = {
 
      val previousCounts: Future[StoredCounts] =  
        statisticsRepository.retrieveLatestCounts(userName)
      val currentCounts: Future[TwitterCounts] =
        twitterService.fetchRelationshipCounts(userName) 

      val counts: Future[(StoredCounts, TwitterCounts)] = for { 
        previous <- previousCounts
        current <- currentCounts
      } yield {
        (previous, current) 
      }

    Future.successful({}) 
  }
```

### Listing 5.17

```
def storeCounts(counts: (StoredCounts, TwitterCounts)): Future[Unit] =
  counts match { case (previous, current) => 
    statisticsRepository.storeCounts(StoredCounts(
      DateTime.now,
      userName,
      current.followersCount,
      current.friendsCount
    ))
  }

def publishMessage(counts: (StoredCounts, TwitterCounts)):
  Future[Unit] =
    counts match { case (previous, current) =>
      val followersDifference = 
        current.followersCount - previous.followersCount
      val friendsDifference =
        current.friendsCount - previous.friendsCount
      def phrasing(difference: Long) =
        if (difference > 0) "gained" else "lost"
      val durationInDays =
        new Period(previous.when, DateTime.now).getDays

  twitterService.postTweet( 
    s"@$userName in the past $durationInDays you have " +
    s"${phrasing(followersDifference)} $followersDifference " +
    s"followers and ${phrasing(followersDifference)} " +
    s"$friendsDifference friends"
  )
}
```

### Listing 5.18

```
// first group of steps: retrieving previous and current counts
val previousCounts: Future[StoredCounts] =
  statisticsRepository.retrieveLatestCounts(userName)
val currentCounts: Future[TwitterCounts] =
  twitterService.fetchRelationshipCounts(userName)

val counts: Future[(StoredCounts, TwitterCounts)] = for { 
  previous <- previousCounts
  current <- currentCounts
} yield {
  (previous, current)
}

// second group of steps: using the counts in order to store them
// and publish a message on Twitter
val storedCounts: Future[Unit] = counts.flatMap(storeCounts) 
val publishedMessage: Future[Unit] = counts.flatMap(publishMessage)

for { 
  _ <- storedCounts 
  _ <- publishedMessage
} yield {} 
```

### Listing 5.19

```
class MongoStatisticsRepository @Inject()
  (reactiveMongo: ReactiveMongoApi) extends StatisticsRepository {
  private val StatisticsCollection = "UserStatistics"

  private lazy val collection =
    reactiveMongo.db.collection[BSONCollection](StatisticsCollection)

  override def storeCounts(counts: StoredCounts)
    (implicit ec: ExecutionContext): Future[Unit] = {
    collection.insert(counts).map { lastError => 
      if(lastError.inError) {
        throw CountStorageException(counts) 
      }
    }
  }

  override def retrieveLatestCounts(userName: String)
    (implicit ec: ExecutionContext): Future[StoredCounts] = {
    val query = BSONDocument("userName" -> userName)
    val order = BSONDocument("_id" -> -1)
    collection
      .find(query)
      .sort(order)
      .one[StoredCounts]
      .map { counts =>
        counts getOrElse StoredCounts(DateTime.now, userName, 0, 0) 
      } recover { 
        case NonFatal(t) => 
          throw CountRetrievalException(userName, t)
        }
  }
}

case class CountRetrievalException(userName: String) 
  extends RuntimeException("Could not read counts for " + userName)

case class CountStorageException(counts: StoredCounts)
  extends RuntimeException
```

### Listing 5.20

```
class DefaultStatisticsService(
  statisticsRepository: StatisticsRepository,
  twitterService: TwitterService) extends StatisticsService {

  // ...

  val result = for {
    _ <- storedCounts
    _ <- publishedMessage
  } yield {}

  result recoverWith {
    case CountStorageException(countsToStore) =>
      retryStoring(countsToStore, attemptNumber = 0)
  } recover {
    case CountStorageException(countsToStore) =>
      throw StatisticsServiceFailed(
        "We couldn't save the statistics to our database. "
        + "Next time it will work!"
      )
    case CountRetrievalException(user, cause) =>
      throw StatisticsServiceFailed(
        "We have a problem with our database. Sorry!", cause
      )
    case TwitterServiceException(message) =>
      throw StatisticsServiceFailed(
        s"We have a problem contacting Twitter: $message"
      )
     case NonFatal(t) =>
       throw StatisticsServiceFailed(
         "We have an unknown problem. Sorry!", t
       )
  }
  
}

class StatisticsServiceFailed(cause: Throwable)
  extends RuntimeException(cause) {
    def this(message: String) = this(new RuntimeException(message))
    def this(message: String, cause: Throwable) =
      this(new RuntimeException(message, cause))
}
object StatisticsServiceFailed {
  def apply(message: String): StatisticsServiceFailed =
    new StatisticsServiceFailed(message)
  def apply(message: String, cause: Throwable):
    StatisticsServiceFailed =
      new StatisticsServiceFailed(message, cause)
}
```

### Listing 5.21

```
private def retryStoring(counts: StoredCounts, attemptNumber: Int)
  (implicit ec: ExecutionContext): Future[Unit] = {
  if (attemptNumber < 3) { 
    statisticsRepository.storeCounts(counts).recoverWith { 
      case NonFatal(t) => retryStoring(counts, attemptNumber + 1) 
    }
  } else {
    Future.failed(CountStorageException(counts)) 
  }
}
```
