### Listing 6.1

```
package actors

import akka.actor.{Actor, ActorLogging, Props}

class StatisticsProvider extends Actor with ActorLogging { 
  def receive = { 
    case message => // do nothing 
  }
}
object StatisticsProvider {
  def props = Props[StatisticsProvider] 
}
```

### Listing 6.2

```
akka {
  loggers = ["akka.event.slf4j.Slf4jLogger"]
  loglevel = "DEBUG"
  logging-filter = "akka.event.slf4j.Slf4jLoggingFilter"
}

```

### Listing 6.3

```
<configuration>

  <!-- ... -->

  <logger name="play" level="INFO" /> 
  <logger name="akka" level="INFO" /> 
  <logger name="application" level="DEBUG" /> 
  <logger name="actors" level="DEBUG" /> 

  <root level="ERROR">
    <appender-ref ref="STDOUT" /> 
  </root>

</configuration>
```

### Listing 6.4

```
package modules

import javax.inject._
import actors.StatisticsProvider
import akka.actor.ActorSystem
import com.google.inject.AbstractModule

class Actors @Inject()(system: ActorSystem)
  extends ApplicationActors { 
  system.actorOf( 
    props = StatisticsProvider.props,
    name = "statisticsProvider"
  )
}

trait ApplicationActors 

class ActorsModule extends AbstractModule { 
  override def configure(): Unit = {
    bind(classOf[ApplicationActors])
      .to(classOf[Actors]).asEagerSingleton 
  }
}
```

### Listing 6.5

```
var reachComputer: ActorRef = _
var storage: ActorRef = _
var followersCounter: ActorRef = _

override def preStart(): Unit = {
  log.info("Starting StatisticsProvider")
  followersCounter = context.actorOf(
    Props[UserFollowersCounter], name = "userFollowersCounter"
  )
  storage = context.actorOf(Props[Storage], name = "storage")
  reachComputer = context.actorOf(
    TweetReachComputer.props(followersCounter, storage),
    name = "tweetReachComputer")
  )

}
```

### Listing 6.6

```
package messages

case class ComputeReach(tweetId: BigInt)
case class TweetReach(tweetId: BigInt, score: Int)

case class FetchFollowerCount(user: String)
case class FollowerCount(user: String, followersCount: Int)

case class StoreReach(tweetId: BigInt, score: Int)
case class ReachStored(tweetId: BigInt)
```

### Listing 6.7

```
class TweetReachComputer(
  userFollowersCounter: ActorRef, storage: ActorRef 
) extends Actor with ActorLogging {
  implicit val executionContext = context.dispatcher 

  var followerCountsByRetweet =
    Map.empty[FetchedRetweet, List[FollowerCount]] 

  def receive = {
    case ComputeReach(tweetId) =>
      fetchRetweets(tweetId, sender()).map { fetchedRetweets => 
        followerCountsByRetweet =
          followerCountsByRetweet + (fetchedRetweets -> List.empty)
        fetchedRetweets.retweeters.foreach { rt =>
          userFollowersCounter ! FetchFollowerCount(tweetId, rt) 
        }
      }

    case count @ FollowerCount(tweetId, _, _) =>
      log.info("Received followers count for tweet {}", tweetId)
      fetchedRetweetsFor(tweetId).foreach { fetchedRetweets =>
        updateFollowersCount(tweetId, fetchedRetweets, count)
      }
    case ReachStored(tweetId) =>
      followerCountsByRetweet.keys
        .find(_.tweetId == tweetId)
        .foreach { key =>
          followerCountsByRetweet =
            followerCountsByRetweet.filterNot(_._1 == key) 
        }
  }

  case class FetchedRetweets(
    tweetId: BigInt, retweeters: List[String], client: ActorRef
  )

  def fetchedRetweetsFor(tweetId: BigInt) =
    followerCountsByRetweet.keys.find(_.tweetId == tweetId)

  def updateFollowersCount(
    tweetId: BigInt,
    fetchedRetweets: FetchedRetweet,
    count: FollowerCount) = {
      val existingCounts = followerCountsByRetweet(fetchedRetweets)
      followerCountsByRetweet =
        followerCountsByRetweet.updated( 
          fetchedRetweets, count :: existingCounts
        )
      val newCounts = followerCountsByRetweet(fetchedRetweets)
      if (newCounts.length == fetchedRetweets.retweeters.length) { 
        log.info(
          "Received all retweeters followers count for tweet {}" +
          ", computing sum", tweetId
        )
        val score = newCounts.map(_.followersCount).sum
        fetchedRetweets.client ! TweetReach(tweetId, score) 
        storage ! StoreReach(tweetId, score)
      }
    }

  def fetchRetweets(tweetId: BigInt, client: ActorRef):
    Future[FetchedRetweets] = ???
}
```

### Listing 6.8

```
import akka.pattern.pipe 

def receive = {
  case ComputeReach(tweetId) =>
    fetchRetweets(tweetId, sender()) pipeTo self 
  case fetchedRetweets: FetchedRetweets => 
    followerCountsByRetweet += fetchedRetweets -> List.empty
    fetchedRetweets.retweets.foreach { rt =>
    userFollowersCounter ! FetchFollowerCount(
      fetchedRetweets.tweetId, rt.user
    )
  }
  ...
}
```

### Listing 6.9

```
case class RetweetFetchingFailed( 
  tweetId: BigInt, cause: Throwable, client: ActorRef
)

def receive = {
  val originalSender = sender()
  case ComputeReach(tweetId) =>
    fetchRetweets(tweetId, sender()).recover { 
      case NonFatal(t) =>
        RetweetFetchingFailed(tweetId, t, originalSender) 
    } pipeTo self 
    ...
}
```

### Listing 6.10

```
class Storage extends Actor with ActorLogging {

  val Database = "twitterService"
  val ReachCollection = "ComputedReach"

  implicit val executionContext = context.dispatcher

  val driver: MongoDriver = new MongoDriver())
  var connection: MongoConnection = _
  var db: DefaultDB = _
  var collection: BSONCollection = _
  obtainConnection()

  override def postRestart(reason: Throwable): Unit = { 
    reason match {
      case ce: ConnectionException => 
        // try to obtain a brand new connection
        obtainConnection()
    }
    super.postRestart(reason)
  }

  override def postStop(): Unit = { 
    connection.close()
    driver.close()
  }

  def receive = {
    case StoreReach(tweetId, score) => // TODO
  }

  private def obtainConnection(): Unit = {
    connection = driver.connection(List("localhost"))
    db = connection.db(Database)
    collection = db.collection[BSONCollection](ReachCollection)
  }
}

case class StoredReach(when: DateTime, tweetId: BigInt, score: Int)
```

### Listing 6.11

```
override def supervisorStrategy: SupervisorStrategy =
  OneForOneStrategy(maxNrOfRetries = 3, withinTimeRange = 2.minutes) { 
    case _: ConnectionException =>
      Restart 
    case t: Throwable =>
      super.supervisorStrategy.decider.applyOrElse(t, _ => Escalate) 
  }
```

### Listing 6.12

```
class StatisticsProvider extends Actor with ActorLogging {
  // ...

  override def preStart(): Unit = {
    // ... initialization of the children actors
    storage = context.actorOf(Props[Storage], name = "storage")
    context.watch(storage) 
  }

  def receive = {
    case reach: ComputeReach =>
      reachComputer forward reach
    case Terminated(terminatedStorageRef) => 
      context.system.scheduler
        .scheduleOnce(1.minute, self, ReviveStorage) 
      context.become(storageUnavailable) 
  }

  def storageUnavailable: Receive = {
    case ComputeReach(_) =>
      sender() ! ServiceUnavailable 
    case ReviveStorage =>
      storage = context.actorOf(Props[Storage], name = "storage") 
      context.unbecome() 
  }

}
object StatisticsProvider {
  case object ServiceUnavailable
  case object ReviveStorage
}
```

### Listing 6.13

```
class TweetReachComputer(
  userFollowersCounter: ActorRef, storage: ActorRef
) extends Actor with ActorLogging with TwitterCredentials {
  // ...
  val retryScheduler: Cancellable = context.system.scheduler.schedule( 
    1.second, 20.seconds, self, ResendUnacknowledged
  )
  override def postStop(): Unit = {
    retryScheduler.cancel() 
  }
  def receive = {
    // ...
    case ResendUnacknowledged =>
      val unacknowledged = followerCountsByRetweet.filterNot { 
        case (retweet, counts) =>
          retweet.retweeters.size != counts.size
      }
      unacknowledged.foreach { case (retweet, counts) =>
        val score = counts.map(_.followersCount).sum
        storage ! StoreReach(retweet.tweetId, score) 
      }
  }

  case object ResendUnacknowledged
}
```

### Listing 6.14

```
class Storage extends Actor with ActorLogging {
  // ...
  var currentWrites = Set.empty[BigInt] 

  def receive = {
    case StoreReach(tweetId, score) =>
    log.info("Storing reach for tweet {}", tweetId)
    if (!currentWrites.contains(tweetId)) { 
      currentWrites = currentWrites + tweetId 
      val originalSender = sender()
      collection
        .insert(StoredReach(DateTime.now, tweetId, score))
        .map { lastError =>
          LastStorageError(lastError, tweetId, originalSender)
      }.recover {
        case _ =>
          currentWrites = currentWrites - tweetId
      } pipeTo self
    }
  case LastStorageError(error, tweetId, client) =>
    if(error.inError) {
      currentWrites = currentWrites - tweetId 
    } else {
      client ! ReachStored(tweetId)
    }
  }
}

object Storage {
  case class LastStorageError(
    error: LastError, tweetId: BigInt, client: ActorRef
  )
}
```

### Listing 6.15

```
class Actors @Inject()(system: ActorSystem)
  extends ApplicationActors {
  Akka.system.actorOf(
    props = StatisticsProvider.props
      .withDispatcher("control-aware-dispatcher"), 
    name = "statisticsProvider"
  )
}
```

### Listing 6.16

```
def receive = {
  // ...
  case TwitterRateLimitReached(reset) =>
    context.system.scheduler.scheduleOnce(
      new Interval(DateTime.now, reset).toDurationMillis.millis, 
      self,
      ResumeService
    )
    context.become({
      case reach @ ComputeReach(_) =>
        sender() ! ServiceUnavailable 
      case ResumeService =>
        context.unbecome() 
    })

case object ResumeService
```

### Listing 6.17

```
val breaker =
  new CircuitBreaker(context.system.scheduler,
    maxFailures = 5, 
    callTimeout = 2.seconds, 
    resetTimeout = 1.minute 
  ).onOpen(
      log.info("Circuit breaker open")
    ).onHalfOpen(
      log.info("Circuit breaker half-open")
    ).onClose(
      log.info("Circuit breaker closed")
    )
```

### Listing 6.18

```
class UserFollowersCounter extends Actor with ActorLogging {
  implicit val ec = context.dispatcher 
  val breaker = ...
  private def fetchFollowerCount(tweetId: BigInt, userId: BigInt):
    Future[FollowerCount] = ... 
  def receive = {
    case FetchFollowerCount(tweetId, user) =>
      breaker
        .withCircuitBreaker(fetchFollowerCount(tweetId, user)) 
        pipeTo sender() 
    }
}
```
