### Listing 7.1

```
db.default.driver="org.postgresql.Driver"
db.default.url="jdbc:postgresql://localhost/chapter7" 
db.default.user=user
db.default.password=secret
db.default.maximumPoolSize = 9 

contexts {
    database {
        fork-join-executor {
          parallelism-max = 9 
        }
    }
}
```

### Listing 7.2

```
package helpers

import play.api.libs.concurrent.Akka
import scala.concurrent.ExecutionContext

object Contexts {
  val database: ExecutionContext =
    Akka.system.dispatchers.lookup("contexts.database")
}
```

### Listing 7.3

```
# --- !Ups 
CREATE TABLE "user" (
    id bigserial PRIMARY KEY,
    email varchar NOT NULL,
    password varchar NOT NULL,
    firstname varchar NOT NULL,
    lastname varchar NOT NULL
);

# --- !Downs 
DROP TABLE "user";
```

### Listing 7.4

```
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<configuration xmlns="http://www.jooq.org/xsd/jooq-codegen-3.7.0.xsd">
  <jdbc>
    <driver>org.postgresql.Driver</driver>
    <url>jdbc:postgresql://localhost/chapter7</url>
    <user>user</user>
    <password>secret</password>
  </jdbc>
  <generator>
    <name>org.jooq.util.ScalaGenerator</name>
    <database>
      <name>org.jooq.util.postgres.PostgresDatabase</name>
      <inputSchema>public</inputSchema>
      <includes>.*</includes>
      <excludes></excludes>
    </database>
    <target>
      <packageName>generated</packageName>
      <directory>app</directory>
    </target>
  </generator>
</configuration>
```

### Listing 7.5

```
val generateJOOQ = taskKey[Seq[File]]("Generate JooQ classes") 

val generateJOOQTask = (baseDirectory, dependencyClasspath in Compile,
  runner in Compile, streams) map { (base, cp, r, s) => 
    toError(r.run( 
      "org.jooq.util.GenerationTool",
      cp.files,
      Array("conf/chapter7.xml"),
      s.log))
    ((base / "app" / "generated") ** "*.scala").get 
}

generateJOOQ <<= generateJOOQTask 
```

### Listing 7.6

```
package modules

import javax.inject.Inject
import com.google.inject.AbstractModule
import org.jooq.SQLDialect
import org.jooq.impl.DSL
import play.api.db.Database
import play.api.libs.Crypto
import generated.Tables._ 

class Fixtures @Inject() (val crypto: Crypto, val db: Database)
  extends DatabaseFixtures{
  db.withTransaction { connection => 
    val sql = DSL.using(connection, SQLDialect.POSTGRES_9_4) 
    if (sql.fetchCount(USER) == 0) { 
      val hashedPassword = crypto.sign("secret")
      sql 
      .insertInto(USER)
      .columns(
        USER.EMAIL, USER.FIRSTNAME, USER.LASTNAME, USER.PASSWORD
      ).values(
        "bob@marley.org", "Bob", "Marley", hashedPassword
      )
      .execute() 
    }
  }
}

trait DatabaseFixtures

class FixturesModule extends AbstractModule {
  override def configure(): Unit = {
    bind(classOf[DatabaseFixtures])
      .to(classOf[Fixtures]).asEagerSingleton
  }
}
```

### Listing 7.7

```
import play.api.db._
import play.api.i18n.{MessagesApi, I18nSupport}
import org.jooq.SQLDialect
import org.jooq.impl.DSL
import generated.Tables._
import generated.tables.records._

class Application(val db: Database, val messagesApi: MessagesApi)
  extends Controller with I18nSupport {
    def login = Action { request =>
      db.withConnection { connection => 
        val sql: DSLContext =
          DSL.using(connection, SQLDialect.POSTGRES_9_4) 
        val users = context.selectFrom[UserRecord](USER).fetch() 
        Ok(users.toString) 
      }
    }
}
```

### Listing 7.8

```
@(form: Form[(String, String)])(implicit messages: Messages) 

@form.globalError.map { error => 
    <p>@error.message</p>
}

@helper.form(controllers.routes.Application.authenticate()) { 
    @helper.inputText(form("email"))
    @helper.inputPassword(form("password"))
    <button type="submit">Login</button>
}
```

### Listing 7.9

```
import play.api.data._
import play.api.data.Forms._

class Application(val db: Database, val messagesApi: MessagesApi)
  extends Controller with I18nSupport {
    // ...

    def login = Action { implicit request =>
      Ok(views.html.login(loginForm))
    }

    val loginForm = Form( 
      tuple(
        "email" -> email,
        "password" -> text
      )
    )
}
```

### Listing 7.10

```
def authenticate = Action { implicit request =>
  loginForm.bindFromRequest.fold( 
    formWithErrors =>
      BadRequest(views.html.login(formWithErrors)), 
    login =>
      db.withConnection { connection =>
        val sql = DSL.using(connection, SQLDialect.POSTGRES_9_4)
        val user = Option(sql
          .selectFrom[UserRecord](USER)
          .where(USER.EMAIL.equal(login._1))
          .and(USER.PASSWORD.equal(crypto.sign(login._2)))
          .fetchOne()) 

        user.map { u =>
          Ok(s"Hello ${u.getFirstname}")
        } getOrElse {
          BadRequest(
            views.html.login(
              loginForm.withGlobalError("Wrong username or password") 
            )
          )
        }
      }
  )
}
```

### Listing 7.11

```
package helpers

import java.sql.Connection
import play.api.Play.current
import scala.concurrent.Future

class Database @Inject() (db: play.api.db.Database) {
  def query[A](block: DSLContext => A): Future[A] = Future { 
    db.withConnection { connection =>
      val sql = DSL.using(connection, SQLDialect.POSTGRES_9_4) 
      block(sql) 
    }
  }(Contexts.database) 
}
```

### Listing 7.12

```
case class AuthenticatedRequest[A]( 
  userId: Long, firstName: String, lastName: String
)

object Authenticated extends ActionBuilder[AuthenticatedRequest] 
  with Results { 

  override def invokeBlock[A] 
    (request: Request[A],
     block: (AuthenticatedRequest[A]) => Future[Result]
    ): Future[Result] = {
    val authenticated = for {
      id <- request.session.get(USER.ID.getName) 
      firstName <- request.session.get(USER.FIRSTNAME.getName)
      lastName <- request.session.get(USER.LASTNAME.getName)
    } yield {
      AuthenticatedRequest[A](id.toLong, firstName, lastName) 
    }

    authenticated.map { authenticatedRequest =>
      block(authenticatedRequest) 
    } getOrElse {
      Future.successful {
        Redirect(routes.Application.login()).withNewSession 
      }
    }
  }
}
```

### Listing 7.13

```
import play.api.cache.Cache

def fetchUser(id: Long) =
  Cache.getAs[UserRecord](id.toString).map { user => 
    Some(user)
  } getOrElse {
    DB.withConnection { connection =>
      val sql = DSL.using(connection, SQLDialect.POSTGRES_9_4)
      val user = Option(
        sql
          .selectFrom[UserRecord](USER)
          .where(USER.ID.equal(id))
          .fetchOne() 
      )
      user.foreach { u =>
        Cache.set(u.getId.toString, u) 
      }
      user
    }
  }
```

### Listing 7.14

```
akka {
  loggers = ["akka.event.slf4j.Slf4jLogger"]
  loglevel = "DEBUG"
  logging-filter = "akka.event.slf4j.Slf4jLoggingFilter"
}
```

### Listing 7.15

```
package actors

import java.net.InetSocketAddress
import akka.actor.{Props, ActorLogging, Actor}
import akka.io.Tcp._
import akka.io.{Tcp, IO}

class SMSServer extends Actor with ActorLogging {

  import context.system 

  IO(Tcp) ! Bind(self, new InetSocketAddress("localhost", 6666)) 

  def receive = {
    case Bound(localAddress) => 
      log.info("SMS server listening on {}", localAddress)

    case CommandFailed(_: Bind) => 
      context stop self

    case Connected(remote, local) =>
      val connection = sender()
      val handler =
        context.actorOf(Props(classOf[SMSHandler], connection)) 
      connection ! Register(handler) 
  }
}
```

### Listing 7.16

```
package actors

import akka.actor.{ActorLogging, Actor}
import akka.io.Tcp._

class SMSHandler(connection: ActorRef)
  extends Actor with ActorLogging {

  def receive = {
    case Received(data) => 
      log.info("Received message: {}", data.utf8String) 
      connection ! Write(data) 
    case PeerClosed => 
      context stop self
  }
}
```

### Listing 7.17

```
package actors

import javax.inject.Inject

import akka.actor.{ActorLogging, Actor, Props}
import com.google.inject.AbstractModule
import helpers.Database
import play.api.libs.concurrent.AkkaGuiceSupport

class SMSService @Inject() (database: Database) 
  extends Actor with ActorLogging {
  // the implementation is left to the reader
}

class SMSServiceModule extends AbstractModule with AkkaGuiceSupport { 
  def configure(): Unit =
    bindActor[SMSService]("sms") 
}
```

### Listing 7.18

```
package actors

import org.joda.time.DateTime

trait Command {
  val phoneNumber: String
}
trait Event {
  val timestamp: DateTime
}

case class RegisterUser(phoneNumber: String, userName: String)
  extends Command
case class UserRegistered(
  phoneNumber: String,
  userName: String,
  timestamp: DateTime = DateTime.now) extends Event

case class InvalidCommand(reason: String)
```

### Listing 7.19

```
package actors

import akka.actor._
import akka.persistence._
import scala.concurrent.duration._

class CQRSCommandHandler extends PersistentActor with ActorLogging {

  override def persistenceId: String = "CQRSCommandHandler"

  override def receiveRecover: Receive = {
    case RecoveryFailure(cause) => 
      log.error(cause, "Failed to recover!")
    case RecoveryCompleted => 
      log.info("Recovery completed")
    case evt: Event => 
      handleEvent(evt)
  }

  override def receiveCommand: Receive = {
      case RegisterUser(phoneNumber, username) =>
        persist(completed(phoneNumber, username))(handleEvent) 
      case command: Command =>
        context.child(command.phoneNumber).map { reference =>
          reference forward command 
        } getOrElse {
          sender() ! "User unknown" 
        }
    }

    def handleEvent(event: Event, recovery: Boolean): Unit =
      event match {
        case registered @ UserRegistered(phoneNumber, userName, _) =>
          context.actorOf( 
            props = Props(
              classOf[ClientCommandHandler], phoneNumber, userName 
            ),
            name = phoneNumber
          )
          if (recoveryFinished) {
            sender() ! registered 
          }
}
```

### Listing 7.20

```
class SMSHandler(connection: ActorRef)
  extends Actor with ActorLogging {

  implicit val timeout = Timeout(2.seconds)
  implicit val ec = context.dispatcher

  lazy val commandHandler = context.actorSelection(
    "akka://application/user/sms/commandHandler"
  )

  val MessagePattern = """[\+]([0-9]*) (.*)""".r 
  val RegistrationPattern = """register (.*)""".r 

  def receive = {
    case Received(data) =>
      log.info("Received message: {}", data.utf8String)
      data.utf8String.trim match {
        case MessagePattern(number, message) =>
          message match {
            case RegistrationPattern(userName) =>
              commandHandler ! RegisterUser(number, userName) 
        case other =>
          log.warning("Invalid message {}", other)
          sender() ! Write(ByteString("Invalid message format\n"))
      }
    case registered: UserRegistered =>
      connection !
        Write(ByteString("Registration successful\n")) 
    case InvalidCommand(reason) =>
      connection ! Write(ByteString(reason + "\n")) 
    case PeerClosed =>
      context stop self
  }
}
```

### Listing 7.21

```
def handleEvent(event: Event): Unit = event match {
  case registered @ UserRegistered(phoneNumber, userName, _) =>
    // ...
    if (recoveryFinished) {
      sender() ! registered
      context.system.eventStream.publish(registered) 
    }
  }
```

### Listing 7.22

```
# --- !Ups

CREATE TABLE "twitter_user" (
  id bigserial PRIMARY KEY,
  created_on timestamp with time zone NOT NULL,
  phone_number varchar NOT NULL,
  twitter_user_name varchar NOT NULL
);

CREATE TABLE "mentions" (
  id bigserial PRIMARY KEY,
  tweet_id varchar NOT NULL,
  user_id bigint NOT NULL,
  created_on timestamp with time zone NOT NULL,
  author_user_name varchar NOT NULL,
  text varchar NOT NULL
);

CREATE TABLE "mention_subscriptions" (
  id bigserial PRIMARY KEY,
  created_on timestamp with time zone NOT NULL,
  user_id bigint NOT NULL
)

# --- !Downs

DROP TABLE "twitter_user";
DROP TABLE "mentions";
DROP TABLE "mention_subscriptions";
```

### Listing 7.23

```
package actors

import java.sql.Timestamp
import akka.actor.{Actor, ActorLogging}
import helpers.Database
import generated.Tables._
import org.jooq.impl.DSL._

class CQRSEventHandler(database: Database)
  extends Actor with ActorLogging {

  override def preStart(): Unit = {
    context.system.eventStream.subscribe(self, classOf[Event]) 
  }

  def receive = {
    case UserRegistered(phoneNumber, userName, timestamp) => // TODO
    case ClientEvent(phoneNumber, userName,
      MentionsSubscribed(timestamp), _) =>
        database.withTransaction { sql =>
          sql.insertInto(MENTION_SUBSCRIPTIONS) 
            .columns(
              MENTION_SUBSCRIPTIONS.USER_ID,
              MENTION_SUBSCRIPTIONS.CREATED_ON
            )
            .select(
              select( 
                TWITTER_USER.ID,
                value(new Timestamp(timestamp.getMillis)) 
              )
              .from(TWITTER_USER)
              .where(
                TWITTER_USER.PHONE_NUMBER.equal(phoneNumber)
                .and(
                  TWITTER_USER.TWITTER_USER_NAME.equal(userName)
                )
              )
          ).execute()
      }
    case ClientEvent(phoneNumber, userName,
      MentionReceived(id, created_on, from, text, timestamp), _) =>
        // TODO
  }
}
```

### Listing 7.24

```
package actors

import akka.actor.Actor
import helpers.Database
import generated.Tables._
import org.jooq.impl.DSL._
import org.jooq.util.postgres.PostgresDataType
import akka.pattern.pipe
import scala.concurrent.Future
import scala.util.control.NonFatal

class CQRSQueryHandler(database: Database) extends Actor {

  implicit val ec = context.dispatcher

  override def receive = {
    case MentionsToday(phoneNumber) =>
      countMentions(phoneNumber).map { count =>
        DailyMentionsCount(count)
      } recover { case NonFatal(t) => 
        QueryFailed
      } pipeTo sender() 
  }

  def countMentions(phoneNumber: String): Future[Int] =
    database.query { sql =>
      sql.selectCount().from(MENTIONS).where(
        MENTIONS.CREATED_ON.greaterOrEqual(currentDate() 
          .cast(PostgresDataType.TIMESTAMP) 
        )
        .and(MENTIONS.USER_ID.equal(
          sql.select(TWITTER_USER.ID)
            .from(TWITTER_USER)
            .where(TWITTER_USER.PHONE_NUMBER.equal(phoneNumber))) 
          )
      ).fetchOne().value1()
    }
}
```
