### Listing 9.1

```
libraryDependencies ++= Seq(
  ws,
  "com.typesafe.play.extras" %% "iteratees-extras" % "1.5.0",
  "com.typesafe.play" %% "play-streams-experimental" % "2.4.2",
  "com.typesafe.akka" % "akka-stream-experimental_2.11" % "1.0"
)
```

### Listing 9.2

```
package services

import javax.inject._
import akka.actor._
import play.api._
import play.api.libs.iteratee._
import play.api.libs.json._
import play.api.libs.oauth._
import play.api.libs.ws._
import play.extras.iteratees._
import scala.concurrent.ExecutionContext

class TwitterStreamService @Inject() (
  ws: WSAPI,
  system: ActorSystem,
  executionContext: ExecutionContext,
  configuration: Configuration
) {
  private def buildTwitterEnumerator( 
    consumerKey: ConsumerKey,
    requestToken: RequestToken,
    topics: Seq[String]
  ): Enumerator[JsObject] = {
    val (iteratee, enumerator) = Concurrent.joined[Array[Byte]] 
    val url =
      "https://stream.twitter.com/1.1/statuses/filter.json"
    implicit val ec = executionContext

    val formattedTopics = topics 
      .map(t => "#" + t)
      .mkString(",")
	
    ws
      .url(url)
      .sign(OAuthCalculator(consumerKey, requestToken))
      .postAndRetrieveStream( 
        Map("track" -> Seq(formattedTopics))
      ) { response =>
        Logger.info("Status: " + response.status)
        iteratee 
      }.map { _ =>
        Logger.info("Twitter stream closed")
      }
	  
    val jsonStream: Enumerator[JsObject] = enumerator &> 
      Encoding.decode() &>
      Enumeratee.grouped(JsonIteratees.jsSimpleObject)

    jsonStream 
  }
}
```

### Listing 9.3

```
import play.api.libs.streams.Streams
import org.reactivestreams.Publisher

private def enumeratorToSource[Out](
  enum: Enumerator[Out]
): Source[Out, Unit] = {
  val publisher: Publisher[Out] =
    Streams.enumeratorToPublisher(enum) 
  Source(publisher) 
}
```

### Listing 9.4

```
def stream(topicsAndDigestRate: Map[String, Int]): 
  Enumerator[JsValue] = { 

  import FanOutShape._

  class SplitByTopicShape[A <: JsObject]( 
    _init: Init[A] = Name[A]("SplitByTopic")
  ) extends FanOutShape[A](_init) {
    protected override def construct(i: Init[A]) =
      new SplitByTopicShape(i)
    val topicOutlets = topicsAndDigestRate.keys.map { topic => 
      topic -> newOutlet[A]("out-" + topic)
    }.toMap
  }

  class SplitByTopic[A <: JsObject] 
    extends FlexiRoute[A, SplitByTopicShape[A]](
      new SplitByTopicShape, Attributes.name("SplitByTopic")
    ) {
    import FlexiRoute._

    override def createRouteLogic(p: PortT) = new RouteLogic[A] { 
      def extractFirstHashTag(tweet: JsObject) = 
        (tweet \ "entities" \ "hashtags")
          .asOpt[JsArray]
          .flatMap { hashtags =>
            hashtags.value.headOption.map { hashtag =>
              (hashtag \ "text").as[String]
            }
          }
      override def initialState =
        State[Any](DemandFromAny(p.topicOutlets.values.toSeq :_*)) { 
          (ctx, _, element) =>
            extractFirstHashTag(element).foreach { topic => 
              p.topicOutlets.get(topic).foreach { port =>
                ctx.emit(port)(element)
              }
            }
            SameState
        }
      override def initialCompletionHandling = eagerClose
    }
  }
  
  Enumerator.empty[JsValue] // we need to continue implementing here
}
```

### Listing 9.5

```
def stream(topicsAndDigestRate: Map[String, Int]):
  Enumerator[JsValue] = {

  // ...

  implicit val fm = ActorMaterializer()(system) 

  val enumerator = buildTwitterEnumerator(
    consumerKey, requestToken, topicsAndDigestRate.keys.toSeq 
  )
  val sink = Sink.publisher[JsValue] 
  val graph = FlowGraph.closed(sink) { implicit builder => out => 
    val in = builder.add(enumeratorToSource(enumerator)) 
    val splitter = builder.add(new SplitByTopic[JsObject]) 
    val groupers = topicsAndDigestRate.map { case (topic, rate) => 
      topic -> builder.add(Flow[JsObject].grouped(rate))
    }
    val taggers = topicsAndDigestRate.map { case (topic, _) => 
      topic -> {
        val t = Flow[Seq[JsObject]].map { tweets =>
          Json.obj("topic" -> topic, "tweets" -> tweets)
        }
        builder.add(t)
      }
    }
    val merger = builder.add(Merge[JsValue](topicsAndDigestRate.size)) 
	
    // TODO: here we will need to wire the graph
  }
  val publisher = graph.run()
  Streams.publisherToEnumerator(publisher)
}
```

### Listing 9.6

```
builder.addEdge(in, splitter.in) 
splitter
  .topicOutlets
  .zipWithIndex
  .foreach { case ((topic, port), index) => 
    val grouper = groupers(topic)
    val tagger = taggers(topic)
    builder.addEdge(port, grouper.inlet) 
    builder.addEdge(grouper.outlet, tagger.inlet) 
    builder.addEdge(tagger.outlet, merger.in(index)) 
  }
builder.addEdge(merger.out, out.inlet) 
```

### Listing 9.7

```
@(topicsAndRate: Map[String, Int], queryString: String)
 (implicit request: RequestHeader)
 <!DOCTYPE html>
<html>
  <head>
    <title>Reactive Tweets</title>
    <script src="//code.jquery.com/jquery-1.11.3.min.js"></script>
    <link
      rel="stylesheet"
      href="http://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/css/
                 bootstrap.min.css">
  </head>
  <body>
  @if(topicsAndRate.nonEmpty) {
    <div class="row">
      @topicsAndRate.keys.map { topic =>
        <div
          id="@topic"
          class="col-md-@{ 12 / (topicsAndRate.size) }"> 
        </div>
      }
    </div>
    <script type="text/javascript">
      function appendTweet(topic, text) { 
        var tweet = document.createElement("p");
        var message = document.createTextNode(text);
        tweet.appendChild(message);
        document.getElementById(topic).appendChild(tweet);
      }
      function connect(url) {
        var tweetSocket = new WebSocket(url);
        tweetSocket.onmessage = function (event) {
          var data = JSON.parse(event.data);
          data.tweets.forEach(function(tweet) {
            appendTweet(data.topic, tweet.text); 
          });
        };
      }
      connect(
        '@routes.Application.stream().webSocketURL()?@queryString'
      );
    </script>
  } else { No topics selected. }
  </body>
</html>
```

### Listing 9.8

```
package controllers

import javax.inject.Inject

import play.api.libs.iteratee._
import play.api.libs.json._
import play.api.mvc._
import services.TwitterStreamService

class Application @Inject() (twitterStream: TwitterStreamService)
  extends Controller {
  def index = Action { implicit request =>
    val parsedTopics = parseTopicsAndDigestRate(request.queryString)
    Ok(views.html.index(parsedTopics, request.rawQueryString))
  }
  def stream = WebSocket.using[JsValue] { request => 
    val parsedTopics = parseTopicsAndDigestRate(request.queryString)
    val out = twitterStream.stream(parsedTopics) 
    val in: Iteratee[JsValue, Unit] = Iteratee.ignore[JsValue] 
    (in, out) 
  }
  private def parseTopicsAndDigestRate(
      queryString: Map[String, Seq[String]]
    ): Map[String, Int] = ??? // TODO
}
```

### Listing 9.9

```
val sleeper = builder.add(Flow[JsValue].map { element =>
  Thread.sleep(5000)
  element
})
builder.addEdge(merger.out, sleeper.inlet)
builder.addEdge(sleeper.outlet, out.inlet)
```
