### Listing 11.1

```
package services

import scala.concurrent.Future

trait RandomNumberService { 
  def generateRandomNumber: Future[Int]
}

class DiceDrivenRandomNumberService(dice: DiceService) 
  extends RandomNumberService {
  override def generateRandomNumber: Future[Int] = dice.throwDice
}

trait DiceService { 
  def throwDice: Future[Int]
}
class RollingDiceService extends DiceService { 
  override def throwDice: Future[Int] =
    Future.successful {
      4 // chosen by fair dice roll.
        // guaranteed to be random.
    }
}
```

### Listing 11.2

```
package services

import org.scalatest.time.{Millis, Span}
import org.scalatest.{ShouldMatchers, FlatSpec}
import org.scalatest.concurrent.ScalaFutures
import scala.concurrent.Future

class DiceDrivenRandomNumberServiceSpec
  extends FlatSpec 
  with ScalaFutures 
  with ShouldMatchers { 

  "The DiceDrivenRandomNumberService" should
    "return a number provided by a dice" in {

    implicit val patienceConfig = 
      PatienceConfig(
        timeout = scaled(Span(150, Millis)), 
        interval = scaled(Span(15, Millis)) 
      )
    val diceService = new DiceService { 
      override def throwDice: Future[Int] = Future.successful(4)
    }
    val randomNumberService = 
      new DiceDrivenRandomNumberService(diceService)

    whenReady(randomNumberService.generateRandomNumber) { result => 
      result shouldBe(4) 
    }
  }
}
```

### Listing 11.3

```
package actors

import actors.RandomNumberComputer._
import akka.actor.{Props, Actor}
import scala.util.Random

class RandomNumberComputer extends Actor {
  def receive = {
    case ComputeRandomNumber(max) =>
      sender() ! RandomNumber(Random.nextInt(max)) 
  }
}

object RandomNumberComputer {
  def props = Props[RandomNumberComputer] 
  case class ComputeRandomNumber(max: Int)
  case class RandomNumber(n: Int)
}
```

### Listing 11.4

```
package actors

import akka.actor.ActorSystem
import akka.testkit._
import scala.concurrent.duration._
import org.scalatest._
import actors.RandomNumberComputer._

class RandomNumberComputerSpec(_system: ActorSystem)
  extends TestKit(_system) 
  with ImplicitSender 
  with FlatSpecLike 
  with ShouldMatchers
  with BeforeAndAfterAll { 

  def this() = this(ActorSystem("RandomNumberComputerSpec")) 

  override def afterAll {
    TestKit.shutdownActorSystem(system) 
  }

  "A RandomNumberComputerSpec" should "send back a random number" in {
    val randomNumberComputer =
      system.actorOf(RandomNumberComputer.props) 
    within(100.millis.dilated) { 
      randomNumberComputer ! ComputeRandomNumber(100)
      expectMsgType[RandomNumber] 
    }
  }
}
```

### Listing 11.5

```
class DiceDrivenRandomNumberServiceSpec
  extends FlatSpec
  with ScalaFutures
  with ShouldMatchers {

  // ...

  it should "be able to cope with problematic dice throws" in {
    val overzealousDiceThrowingService = new DiceService { 
      val counter = new AtomicInteger()
      override def throwDice: Future[Int] = {
        val count = counter.incrementAndGet()
        if(count % 2 == 0) {
          Future.successful(4)
        } else {
          Future.failed(new RuntimeException(
            "Dice fell of the table and the cat won't give it back"
          ))
        }
      }
    }

    val randomNumberService =
      new DiceDrivenRandomNumberService(
        overzealousDiceThrowingService
      )

    whenReady(randomNumberService.generateRandomNumber) { result =>
      result shouldBe(4) 
     }
  }
}
```

### Listing 11.6

```
import scala.util.control.NonFatal
import scala.concurrent.ExecutionContext.Implicits._

class DiceDrivenRandomNumberService(dice: DiceService)
  extends RandomNumberService {
  override def generateRandomNumber: Future[Int] =
    dice.throwDice.recoverWith { 
      case NonFatal(t) => generateRandomNumber 
    }
}
```

### Listing 11.7

```
it should "fail when the maximum is a negative number" in {

    class StepParent(target: ActorRef) extends Actor { 
      override def supervisorStrategy: SupervisorStrategy =
        OneForOneStrategy() { 
          case t: Throwable =>
            target ! t 
            Restart
        }
      def receive = {
        case props: Props =>
          sender ! context.actorOf(props) 
      }
    }

    val parent = system.actorOf(
      Props(new StepParent(testActor)), name = "stepParent" 
    )
    parent ! RandomNumberComputer.props 
    val actorUnderTest = expectMsgType[ActorRef] 
    actorUnderTest ! ComputeRandomNumber(-1) 
    expectMsgType[IllegalArgumentException] 
  }
```

### Listing 11.8

```
package actors

import actors.RandomNumberFetcher._
import akka.actor.{Props, Actor}
import play.api.libs.json.{JsArray, Json}
import play.api.libs.ws.WSClient
import scala.concurrent.Future
import akka.pattern.pipe

class RandomNumberFetcher(ws: WSClient) extends Actor {
  implicit val ec = context.dispatcher

  def receive = {
    case FetchRandomNumber(max) =>
      fetchRandomNumber(max).map(RandomNumber) pipeTo sender() 
  }

  def fetchRandomNumber(max: Int): Future[Int] =
    ws 
      .url("https://api.random.org/json-rpc/1/invoke")
      .post(Json.obj(
        "jsonrpc" -> "2.0",
        "method" -> "generateIntegers",
        "params" -> Json.obj(
          "apiKey" -> "00000000-0000-0000-0000-000000000000", 
          "n" -> 1,
          "min" -> 0,
          "max" -> max,
          "replacement" -> true,
          "base" -> 10
        ),
        "id" -> 42
      )).map { response =>
        (response.json \ "result" \ "random" \ "data")
          .as[JsArray]
          .value
          .head
          .as[Int] 
      }
}

object RandomNumberFetcher {
  def props(ws: WSClient) = Props(classOf[RandomNumberFetcher], ws)
  case class FetchRandomNumber(max: Int)
  case class RandomNumber(n: Int)
}
```

### Listing 11.9

```
package controllers

import javax.inject.Inject
import scala.concurrent.duration._
import scala.concurrent.ExecutionContext
import play.api.mvc._
import akka.actor._
import akka.util.Timeout
import akka.pattern.ask
import actors._
import actors.RandomNumberFetcher._
import play.api.libs.ws.WSClient

class Application @Inject() (ws: WSClient,
                             ec: ExecutionContext,
                             system: ActorSystem) extends Controller { 

  implicit val executionContext = ec
  implicit val timeout = Timeout(2000.millis) 

  val fetcher = system.actorOf(RandomNumberFetcher.props(ws)) 

  def index = Action { implicit request =>
    Ok(views.html.index())
  }

  def compute = Action.async { implicit request =>
    (fetcher ? FetchRandomNumber(10)).map { 
      case RandomNumber(r) =>
        Redirect(routes.Application.index())
          .flashing("result" -> s"The result is $r") 
      case other =>
        InternalServerError 
    }
  }
}
```

### Listing 11.10

```
@()(implicit flash: Flash) 
@main("Welcome") {
    @flash.get("result").map { result =>
        <p>@result</p> 
    }
    @helper.form(routes.Application.compute()) {
        <button type="submit">Get random number</button> 
    }
}
```

### Listing 11.11

```
setUp(
  scn.inject(
    nothingFor(4 seconds), 
    rampUsers(50) over(10 seconds), 
    atOnceUsers(10), 
    constantUsersPerSec(2) during(15 seconds) randomized, 
    splitUsers(50) into (
      rampUsers(10) over(10 seconds)
    ) separatedBy(5 seconds) 
  ).protocols(httpProtocol)
)
```

### Listing 11.12

```
[Credentials]
aws_access_key_id = <your access key id>
aws_secret_access_key = <your secret access key>
```
